from truegaze.plugins.adobe_mobile_sdk import  AdobeMobileSdkPlugin

# List of active plugins - when developing a new plugin, it should be added here.
# BasePlugin should never be added to this list.
ACTIVE_PLUGINS = [
    AdobeMobileSdkPlugin
]


#
# Copyright (c) 2019 Nightwatch Cybersecurity.
#
# This file is part of truegaze
# (see https://github.com/nightwatchcybersecurity/truegaze).
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
import json
import re

import click
import jmespath

from truegaze.plugins.base import BasePlugin
from truegaze.utils import TruegazeUtils

# Regex pattern for the configuration file
CONFIG_FILE_PATTERN =\
    re.compile(r'assets/ADBMobileConfig\.json|ADBMobileConfig\.json|.*/ADBMobileConfig\.json')


#
# Plugin to support detection of incorrect SSL configuration in the Adobe Mobile SDK. This plugin will not
# search for config files that are not named in a standard fashion.
#
# Adobe documentation including field definitions can be found here:
# https://docs.adobe.com/content/help/en/mobile-services/android/configuration-android/json-config.html
# https://docs.adobe.com/content/help/en/mobile-services/ios/config-ios/json-config.html
#


class AdobeMobileSdkPlugin(BasePlugin):
    name = 'AdobeMobileSdk'
    desc = 'Detection of incorrect SSL configuration\nin the Adobe Mobile SDK'
    supports_android = True
    supports_ios = True

    # Main scanning method
    def scan(self):
        # On Android, the config file is usually in the assets folder but can be placed elsewhere.
        # On iOS the configuration file can be anywhere.

        # Search all paths for the config file
        paths = AdobeMobileSdkPlugin.get_paths(self.zip_file)
        if len(paths) == 0:
            click.echo('-- Cannot find the "ADBMobileConfig.json" file, skipping')
            return

        # Loop through files, parse the JSON and analyze
        click.echo('-- Found ' + str(len(paths)) + ' configuration file(s)')
        for path in paths:
            click.echo('-- Scanning "' + path + "'")

            # Try to parse the data
            parsed_data = AdobeMobileSdkPlugin.parse_data(self.zip_file, path)
            if not parsed_data:
                click.echo('---- ERROR: Unable to parse config file - will skip. File: ' + path)
                continue

            # Check the SSL setting
            if not AdobeMobileSdkPlugin.is_ssl_setting_correct(parsed_data):
                click.echo('---- FOUND: The ["analytics"]["ssl"] setting is missing or false - SSL is not being used')

            # Check the POI URL
            poi_url = AdobeMobileSdkPlugin.get_vulnerable_poi_url(parsed_data)
            if poi_url:
                click.echo('---- FOUND: The ["remotes"]["analytics.poi"] URL doesn\'t use SSL: ' + poi_url)

            # Check the messages URL
            messages_url = AdobeMobileSdkPlugin.get_vulnerable_poi_url(parsed_data)
            if messages_url:
                click.echo('---- FOUND: The ["remotes"]["messages"] URL doesn\'t use SSL: ' + messages_url)

            # Checking postback URLs in the messages section
            postback_urls = AdobeMobileSdkPlugin.get_vulnerable_postback_urls(parsed_data)
            for url in postback_urls:
                click.echo('---- FOUND: A "templateurl" in ["messages"]["payload"] doesn\'t use SSL: ' + url)

    # Gets paths for the configuration file from the ZIP File
    @staticmethod
    def get_paths(zip_file):
        return TruegazeUtils.get_matching_paths_from_zip(zip_file, CONFIG_FILE_PATTERN)

    # Parses the config file from a given path
    @staticmethod
    def parse_data(zip_file, path):
        data = zip_file.read(path)
        try:
            parsed_data = json.loads(data)
        except json.JSONDecodeError:
            return None
        return parsed_data

    # Checks if the SSL setting is present, and set to true, Otherwise, the default is false.
    @staticmethod
    def is_ssl_setting_correct(parsed_data):
        ssl_setting = jmespath.search('analytics.ssl', parsed_data)
        if ssl_setting and type(ssl_setting) == bool and ssl_setting is True:
            return True

        return False

    # Extracts and returns the POI URL if it is vulnerable
    @staticmethod
    def get_vulnerable_poi_url(parsed_data):
        poi_url = jmespath.search('remotes."analytics.poi"', parsed_data)
        if poi_url and not poi_url.strip().startswith('https://'):
            return poi_url.strip()

        return None

    # Extracts and returns the Messages URL if it is vulnerable
    @staticmethod
    def get_vulnerable_messages_url(parsed_data):
        messages_url = jmespath.search('remotes.messages', parsed_data)
        if messages_url and not messages_url.strip().startswith('https://'):
            return messages_url.strip()

        return None

    # Extracts and returns a list of vulnerable template URLs in the messages postbacks section.
    @staticmethod
    def get_vulnerable_postback_urls(parsed_data):
        vulnerable_urls = []
        postback_urls = jmespath.search('messages[].payload.templateurl', parsed_data)
        if postback_urls:
            for url in postback_urls:
                if url and not url.strip().startswith('https://'):
                    vulnerable_urls.append(url.strip())

        return vulnerable_urls



#
# Copyright (c) 2019 Nightwatch Cybersecurity.
#
# This file is part of truegaze
# (see https://github.com/nightwatchcybersecurity/truegaze).
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
import sys

import click
from beautifultable import BeautifulTable

from truegaze.plugins import ACTIVE_PLUGINS
from truegaze.utils import TruegazeUtils

@click.group()
def cli():
    """
    truegaze - A static analysis tool for Android and iOS applications focusing on security issues
    outside the source code such as resource strings, third party libraries and configuration files.

    Copyright (c) 2019 Nightwatch Cybersecurity.
    Source code: https://github.com/nightwatchcybersecurity/truegaze
    """


@cli.command('list')
def list_plugins():
    """List supported plugins"""
    click.echo("Total active plugins: " + str(len(ACTIVE_PLUGINS)))

    # Get information out of the plugin list into a table
    table = BeautifulTable()
    table.column_headers = ['Name', 'Description', 'Android', 'iOS']
    for plugin in ACTIVE_PLUGINS:
        table.append_row([plugin.name, plugin.desc, plugin.supports_android, plugin.supports_ios])
    click.echo(table)


@cli.command('scan')
@click.argument('filename', type=click.Path(exists=True, dir_okay=False))
def scan(filename):
    """Scan the provided file for vulnerabilities"""

    # Try to open the provided file as a ZIP, fail otherwise
    zip_file = TruegazeUtils.open_file_as_zip(filename)
    if zip_file is None:
        click.echo('ERROR: Unable to open file - please check to make sure it is an APK or IPA file')
        sys.exit(-1)

    # Detect manifest
    is_android = False
    is_ios = False
    android_manifest = TruegazeUtils.get_android_manifest(zip_file)
    ios_manifest = TruegazeUtils.get_ios_manifest(zip_file)

    # Set flags, error out if no manifest is found
    if android_manifest:
        click.echo('Identified as an Android application via a manifest located at: ' + android_manifest)
        is_android = True
    elif ios_manifest:
        click.echo('Identified as an iOS application via a manifest located at: ' + ios_manifest)
        is_ios = True
    else:
        click.echo('ERROR: Unable to identify the file as an Android or iOS application')
        sys.exit(-2)

    # Pass the filename to the individual modules for scanning
    for PLUGIN in ACTIVE_PLUGINS:
        click.echo('Scanning using the "' + PLUGIN.name + '" plugin')
        instance = PLUGIN(zip_file, is_android, is_ios)

        # Show error if OS is not supported
        if instance.is_os_supported:
            instance.scan()
        else:
            click.echo('-- OS is not supported by this plugin, skipping')

    click.echo("Done!")

@cli.command('version')
def version():
    """Displays current version"""
    click.echo("Current version: v" + TruegazeUtils.get_version())


if __name__ == '__main__':
    cli()